@@ -1,5 +1,5 @@
 
-.as.pedigree = function(x, deceased=numeric(0), aff2=NULL) {
+.as.kinship2_pedigree = function(x, deceased=numeric(0), aff2=NULL) {
 	ped = relabel(x$pedigree, x$orig.ids) 
 	AFF = ped[, 'AFF']; AFF[AFF == 0] = NA; AFF = AFF - 1 #kinship2:::plot.pedigree uses aff codes 1=aff, 0=unaff, NA=unknown
 	#if (all(AFF==0)) AFF = AFF+1 # because of a bug(?) in kinship2
@@ -13,11 +13,11 @@
 	pedigree(id=ped[,'ID'], dadid=ped[,'FID'], momid=ped[,'MID'], sex=ped[,'SEX'], affected=AFF, status=status)
 }
 
-plot.linkdat <- function(x, marker=NULL, alleles=NULL, sep="/", missing="-", 
+plot.linkdat <- function(x, marker=NULL, alleles=NULL, sep="/", missing="-", skip.empty.genotypes=FALSE,
 	id.labels=x$orig.ids, title=paste("Family", x$famid), available=FALSE, col = 1,
-	deceased=numeric(0), starred=numeric(0), aff2=NULL, margins=c(4.1, 1, 4.1, 1), ...) {
+	deceased=numeric(0), starred=numeric(0), aff2=NULL, margins=c(0.6, 1, 4.1, 1), ...) {
 	
-	pedigree = .as.pedigree(x, deceased=deceased, aff2=aff2)
+	pedigree = .as.kinship2_pedigree(x, deceased=deceased, aff2=aff2)
 	
 	if(available) cols = ifelse(x$orig.ids %in% x$available, 2, 1)
 	else cols = col
@@ -34,13 +34,14 @@ plot.linkdat <- function(x, marker=NULL, alleles=NULL, sep="/", missing="-",
 		else m = x$markerdata[marker]
 		chrom = ifelse(all(sapply(m, function(mm) identical(23L, as.integer(attr(mm, 'chrom'))))), 'X', 'AUTOSOMAL')
 		gg = .prettyMarkers(m, alleles=alleles, sep=sep, missing=missing, singleCol=TRUE, sex=x$pedigree[, 'SEX'])
-		geno = gg[,1]
-		for(i in seq_along(m)[-1]) geno = paste(geno, gg[, i], sep="\n")
+		geno = apply(gg, 1, paste, collapse="\n")
+      if (skip.empty.genotypes) geno[rowSums(do.call(cbind, m))==0] = ""
 		if(is.null(strid) || !any(nzchar(strid))) strid = geno else strid = paste(strid, geno, sep="\n")
 	}	
-	#par(xpd=T)
-	plot(pedigree, id=strid, col=cols, mar=margins, ...)
+	oldmar = par(mar=margins)  # without this, par() does not equal 'margins'...(why??) Needed for centered title.
+   plot(pedigree, id=strid, col=cols, mar=margins, ...)
 	if(!is.null(title)) title(title)
+   par(oldmar)
 }
 
 
